@@ -271,7 +271,11 @@ RGWAS.epistasis <- function(pheno, geno, ZETA = NULL, covariate = NULL, covariat
     X.now <- as.matrix(X[not.NA, ])
     ZETA.now <- lapply(ZETA, function(x) list(Z = x$Z[not.NA, ], K = x$K))
 
-    M.now <- Z.A[not.NA, ] %*% M
+    if (sum(is.na(M)) == 0) {
+      M.now <- Z.A[not.NA, ] %*% M
+    } else {
+      M.now <- M[apply(Z.A[not.NA, ], 1, function(x) which(x == 1)), ]
+    }
 
 
     #### Calculate LL for the null hypothesis at first ####
